import pandas as pd


def _get_storage_class_values(cls):
    return [getattr(cls, c) for c in cls.__dict__ if not c.startswith("_")]


class IdxTypes:
    nunmeric = 'numeric'
    datetime = 'datetime'
    mixed = 'mixed'
    other = 'other'


def get_indextype(obj):
    if _is_dtIndex_like(obj):
        return IdxTypes.datetime

    if _is_numIndex_like(obj):
        return IdxTypes.nunmeric

    if _is_pdIndex_like(obj):
        return IdxTypes.other

    idxtypes = _get_storage_class_values(IdxTypes)
    for itype in idxtypes:
        if obj == itype:
            return itype

    raise ValueError(f"{type(obj)} is not a indextype nor any known subtype of pd.Index")


def _is_dtIndex_like(i):
    if isinstance(i, pd.DatetimeIndex):
        return True
    try:
        if i == pd.DatetimeIndex:
            return True
    except TypeError:
        return False


def _is_numIndex_like(i):
    tup = (pd.RangeIndex, pd.Int64Index, pd.UInt64Index, pd.Float64Index)
    if isinstance(i, tup):
        return True
    # was a pd.xxxIndex was given
    for it in tup:
        try:
            if it == i:
                return True
        except TypeError:
            pass
    return False


def _is_pdIndex_like(i):
    """See here:
    https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.Index.html#pandas.Index
    """
    if isinstance(i, pd.Index):
        return True
    tup = (pd.RangeIndex, pd.CategoricalIndex, pd.MultiIndex, pd.IntervalIndex,
           pd.DatetimeIndex, pd.TimedeltaIndex,
           pd.PeriodIndex, pd.Int64Index, pd.UInt64Index, pd.Float64Index)
    # was a pd.xxxIndex was given
    for it in tup:
        try:
            if it == i:
                return True
        except TypeError:
            pass
    return False


